/***********************************************************************
 * A 4-bit Johnson Counter with zero delays and with feedback.
 *
 * USE CONDITIONAL COMPILATION FOR DIFFERENT VERSIONS OF THE MODEL
 * `define ONE_PROC   //to compile with 4 flip-flops in 1 procedure
 * `define FOUR_PROC  //to compile with each flip-flops in a
 *                    //separate procedure
 *
 **********************************************************************/

module jcounter
(output logic  q1, q2, q3, q4,
 input  logic  clk, rst_);

  timeunit 1ns; timeprecision 1ns;

//`define ONE_PROC   // or invoke simulation with +define+ONE_PROC
//`define FOUR_PROC  // or invoke simulation with +define+FOUR_PROC

`ifdef ONE_PROC      // MODEL THE COUNTER AS A SINGLE PROCEDURE
  always_ff @(posedge clk or negedge rst_) begin
    if (!rst_) begin      // reset the counter
      q1 <= 1'b0;
      q2 <= 1'b0;
      q3 <= 1'b0;
      q4 <= 1'b0;
    end
    else begin           // shift the count
      q1 <= ~q4;
      q3 <= q2;
      q2 <= q1;
      q4 <= q3;
    end
  end

`else
`ifdef FOUR_PROC     // MODEL EACH FLIP-FLOP AS A SEPARATE PROCEDURE
  always_ff @(posedge clk or negedge rst_) begin
    if (!rst_)        // reset the counter bit
      q1 <= 1'b0;
    else              // shift
      q1 <= ~q4;
  end

  always_ff @(posedge clk or negedge rst_) begin
    if (!rst_)        // reset the counter bit
      q3 <= 1'b0;
    else              // shift
      q3 <= q2;
  end

  always_ff @(posedge clk or negedge rst_) begin
    if (!rst_)        // reset the counter bit
      q2 <= 1'b0;
    else              // shift
      q2 <= q1;
  end

  always_ff @(posedge clk or negedge rst_) begin
    if (!rst_)        // reset the counter bit
      q4 <= 1'b0;
    else              // shift
      q4 <= q3;
  end

`else
  initial
    $display("\nERROR IN jcounter_solution.v: NO CONDITIONAL COMPILATION MACROS WERE SET.\n");
`endif  // FOUR_PROC
`endif  // ONE_PROC

endmodule: jcounter

