/***********************************************************************
 * Test bench for the Johnson Counter model.
 *
 **********************************************************************/

module jcounter_test
(output logic  rst_,             // input to counter
 input  logic  q1, q2, q3, q4,   // outputs from counter
 input  logic  clk);             // from top module

  timeunit 1ns; timeprecision 1ns;

  initial begin // input stimulus
    rst_ = 1'b1;
    repeat (2) @(negedge clk) ;           // let simulation stablilize
      $display("\nTesting reset, should not shift ...");
      rst_ = 1'b0;                        // assert reset
    repeat (5) @(posedge clk) ;           // run 5 clock cycles
    @(negedge clk)
      rst_ = 1'b1;                        // de-assert reset
      $display("\nTesting shift ...");
    repeat (12) @(posedge clk) ;          // run 12 clock cycles
    @(negedge clk)
      $stop();                              // end of tests
    @(negedge clk)
      $finish();                            // exit simulation
  end

  initial begin  // response checking (no self-test)
    $timeformat(-9, 0, " ns", 9);
    $display("            INPUTS  |     OUTPUTS   ");
    $display("  time     clk  rst | q1  q2  q3  q4");
    $display("---------  ---  --- | --  --  --  --");
    forever @(posedge clk)
      $strobe("%t   %b    %b  |  %b   %b   %b   %b",
              $time, clk, rst_, q1, q2, q3, q4);
  end
endmodule: jcounter_test
