/***********************************************************************
 * An RTL Arithmetic Logic Unit (ALU) model.
 *
 **********************************************************************/

module alu
  import alu_opcodes::*;          // import ALU opcode definitions
 (output logic [3:0]  result,
  input  logic [3:0]  a, b,
  input  opcode_t     opcode
 );
  timeunit 1ns; timeprecision 1ns;

  // ADD a combinational logic always procedure
  // to perform the 8 alu operations
// START OF SOLUTION
  int max;  // local temporary variable
  always_comb  begin
    assert #0 (!$isunknown(opcode)) else $error(" Invalid opcode value %b in ALU instance %m", opcode);
    max = (|b[3:2]==0)? b[1:0] : 4;  // shift/rotate b times, up to a max of 4 times
    case (opcode)
      INVERT_A   :  result = ~a;
      NEGATE_A   :  result = -a;
      INCREMENT_A:  result = a + 1'b1;
      A_PLUS_B   :  result = a + b;
      A_MINUS_B  :  result = a - b;
      A_XORED_B  :  result = a ^ b;
      A_SHIFTED_B:  result = a >> max;      // shift b times; max of 4
      A_ROTATED_B:  result = {a,a} >> max;  // rotate b times; max of 4
    endcase
  end
// END OF SOLUTION
endmodule: alu
