
// Extra Credit lab for "SystemVerilog Object Oriented Verification"
// Using a "solve before" constraint
//
module fifty_fifty();

  parameter TRANS_COUNT = 100;

  class Transaction;
    typedef enum {ZERO, ANY} mode_t;
    rand mode_t    mode;
    rand bit [7:0] data;
    real           zeros;

    function void count_zeros ();
      if ((this.mode == ZERO) && (this.data == 0)) zeros++;
    endfunction

    function void print_values ();
      $display("mode=%-4s   data=%0d", this.mode.name, this.data);
    endfunction

    function void print_summary ();
      $display("\nPercent of transactions with mode = ZERO and data == 0 is %0d%% \n",
               (this.zeros/TRANS_COUNT)*100);
    endfunction

  endclass: Transaction

  Transaction tx;

  initial begin
    tx = new;
    repeat(TRANS_COUNT) begin
       if (!tx.randomize()) $error;
       tx.count_zeros;
       tx.print_values;
    end
    tx.print_summary;
    $finish();
  end
endmodule: fifty_fifty

