/***********************************************************************
 * A SystemVerilog top-level netlist to connect testbench to DUT
 *
 **********************************************************************/

module top();
  timeunit 1ns; timeprecision 1ns;

  initial $timeformat(-9,0, "ns", 3);

  // user-defined types are defined in instr_register_pkg.sv
  import instr_register_test_pkg::*;

  // clock variables
  logic clk;
  int num_trans = 3;
  int ver_dec;
  
  BaseTest test_h;
  
  tb_ifc io (.clk(clk));

  // connect interface to testbench removed
  //instr_register_test test (.io(io));

  // instantiate design and connect ports
  instr_register dut (
    .clk(clk),
    .load_en(io.load_en),
    .reset_n(io.reset_n),
    .operand_a(io.operand_a),
    .operand_b(io.operand_b),
    .opcode(io.opcode),
    .write_pointer(io.write_pointer),
    .read_pointer(io.read_pointer),
    .instruction_word(io.instruction_word)
   );

  // clock oscillator
  initial begin
    clk <= 0;
    forever #5  clk = ~clk;
  end

  initial begin
    if ($value$plusargs("num_trans=%d", num_trans))
      $display("\n+num_trans=%0d found", num_trans);
    else
      $display("\nNo +num_trans found, using default of %0d", num_trans);
    if ($value$plusargs("verbosity=%d", ver_dec))begin
      verbosity = verbosity_t'(ver_dec);
    end
    $display("\n+verbosity is %s", verbosity.name);
    instr_register_test_pkg::v_io = io;
    test_h = new();
    test_h.build();
    test_h.connect();
    test_h.reset();
    test_h.run(num_trans);
    $display("Injecting Divide_by_zero pattern"); // New code for #1 only
    test_h.gen_h.trn_h = DivideByZero::new();
    test_h.run(1);
    test_h.check(num_trans + 1);
    test_h.report();
    $finish;
  end

endmodule: top
